"use strict";

var __awaiter = undefined && undefined.__awaiter || function (thisArg, _arguments, Promise, generator) {
    return new Promise(function (resolve, reject) {
        generator = generator.call(thisArg, _arguments);
        function cast(value) {
            return value instanceof Promise && value.constructor === Promise ? value : new Promise(function (resolve) {
                resolve(value);
            });
        }
        function onfulfill(value) {
            try {
                step("next", value);
            } catch (e) {
                reject(e);
            }
        }
        function onreject(value) {
            try {
                step("throw", value);
            } catch (e) {
                reject(e);
            }
        }
        function step(verb, value) {
            var result = generator[verb](value);
            result.done ? resolve(result.value) : cast(result.value).then(onfulfill, onreject);
        }
        step("next", void 0);
    });
};
require('babel-polyfill');
var Promise = require('bluebird');
var _ = require('lodash');
var deps_1 = require('./deps');
var cache_1 = require('./cache');
var helpers = require('./helpers');
var instance_1 = require('./instance');
let loaderUtils = require('loader-utils');
let cachePromise = Promise.promisify(cache_1.cache);
function loader(text) {
    compiler.call(undefined, this, text);
}
let externalsInvocation;
function compiler(webpack, text) {
    return __awaiter(this, void 0, Promise, regeneratorRuntime.mark(function _callee2() {
        var options, instanceName, instance, state, callback, fileName, resolver, depsInjector, applyDeps, doUpdate, wasChanged, compiledModule, transformation, transform, resultText, resultSourceMap;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
            while (1) switch (_context2.prev = _context2.next) {
                case 0:
                    if (webpack.cacheable) {
                        webpack.cacheable();
                    }
                    options = loaderUtils.parseQuery(webpack.query);
                    instanceName = options.instanceName || 'default';
                    instance = instance_1.ensureInstance(webpack, options, instanceName);
                    state = instance.tsState;
                    callback = webpack.async();
                    fileName = state.normalizePath(webpack.resourcePath);
                    resolver = deps_1.createResolver(webpack._compiler.options.externals, webpack.resolve);
                    depsInjector = {
                        add: depFileName => {
                            webpack.addDependency(depFileName);
                        },
                        clear: webpack.clearDependencies.bind(webpack)
                    };
                    applyDeps = _.once(() => {
                        depsInjector.clear();
                        depsInjector.add(fileName);
                        state.fileAnalyzer.dependencies.applyCompiledFiles(fileName, depsInjector);
                        if (state.options.reEmitDependentFiles) {
                            state.fileAnalyzer.dependencies.applyChain(fileName, depsInjector);
                        }
                    });

                    if (!(options.externals && !instance.externalsInvoked)) {
                        _context2.next = 20;
                        break;
                    }

                    if (!externalsInvocation) {
                        _context2.next = 16;
                        break;
                    }

                    _context2.next = 14;
                    return externalsInvocation;

                case 14:
                    _context2.next = 19;
                    break;

                case 16:
                    externalsInvocation = options.externals.map(external => __awaiter(this, void 0, Promise, regeneratorRuntime.mark(function _callee() {
                        return regeneratorRuntime.wrap(function _callee$(_context) {
                            while (1) switch (_context.prev = _context.next) {
                                case 0:
                                    _context.next = 2;
                                    return state.fileAnalyzer.checkDependencies(resolver, external);

                                case 2:
                                case "end":
                                    return _context.stop();
                            }
                        }, _callee, this);
                    })));
                    _context2.next = 19;
                    return Promise.all(externalsInvocation);

                case 19:
                    instance.externalsInvoked = true;

                case 20:
                    instance.compiledFiles[fileName] = true;
                    doUpdate = false;

                    if (instance.options.useWebpackText) {
                        if (state.updateFile(fileName, text, true)) {
                            doUpdate = true;
                        }
                    }
                    _context2.prev = 23;
                    _context2.next = 26;
                    return state.fileAnalyzer.checkDependencies(resolver, fileName);

                case 26:
                    wasChanged = _context2.sent;

                    if (wasChanged || doUpdate) {
                        state.updateProgram();
                    }

                    if (instance.options.usePrecompiledFiles) {
                        compiledModule = cache_1.findCompiledModule(fileName);
                    }
                    transformation = null;

                    if (!compiledModule) {
                        _context2.next = 35;
                        break;
                    }

                    state.fileAnalyzer.dependencies.addCompiledModule(fileName, compiledModule.fileName);
                    transformation = {
                        text: compiledModule.text,
                        map: JSON.parse(compiledModule.map)
                    };
                    _context2.next = 43;
                    break;

                case 35:
                    transform = function transform() {
                        let resultText;
                        let resultSourceMap;
                        let output = state.emit(fileName);
                        let result = helpers.findResultFor(output, fileName);
                        if (result.text === undefined) {
                            throw new Error('No output found for ' + fileName);
                        }
                        resultText = result.text;
                        resultSourceMap = JSON.parse(result.sourceMap);
                        resultSourceMap.sources = [fileName];
                        resultSourceMap.file = fileName;
                        resultSourceMap.sourcesContent = [text];
                        if (instance.options.useBabel) {
                            let defaultOptions = {
                                inputSourceMap: resultSourceMap,
                                filename: fileName,
                                sourceMap: true
                            };
                            let babelResult = instance.babelImpl.transform(resultText, defaultOptions);
                            resultText = babelResult.code;
                            resultSourceMap = babelResult.map;
                        }
                        return {
                            text: resultText,
                            map: resultSourceMap
                        };
                    };

                    if (!instance.options.useCache) {
                        _context2.next = 42;
                        break;
                    }

                    _context2.next = 39;
                    return cachePromise({
                        source: text,
                        identifier: instance.cacheIdentifier,
                        directory: instance.options.cacheDirectory,
                        options: webpack.query,
                        transform: transform
                    });

                case 39:
                    transformation = _context2.sent;
                    _context2.next = 43;
                    break;

                case 42:
                    transformation = transform();

                case 43:
                    resultText = transformation.text;
                    resultSourceMap = transformation.map;

                    if (resultSourceMap) {
                        resultSourceMap.sources = [fileName];
                        resultSourceMap.file = fileName;
                        resultSourceMap.sourcesContent = [text];
                    }
                    try {
                        callback(null, resultText, resultSourceMap);
                    } catch (e) {
                        console.error('Error in bail mode:', e);
                        process.exit(1);
                    }
                    _context2.next = 52;
                    break;

                case 49:
                    _context2.prev = 49;
                    _context2.t0 = _context2["catch"](23);

                    callback(_context2.t0, helpers.codegenErrorReport([_context2.t0]));

                case 52:
                    _context2.prev = 52;

                    applyDeps();
                    return _context2.finish(52);

                case 55:
                case "end":
                    return _context2.stop();
            }
        }, _callee2, this, [[23, 49, 52, 55]]);
    }));
}
class ForkCheckerPlugin {
    apply(compiler) {
        compiler.plugin("watch-run", function (params, callback) {
            compiler._tsFork = true;
            callback();
        });
    }
}
loader.ForkCheckerPlugin = ForkCheckerPlugin;
module.exports = loader;
//# sourceMappingURL=index.js.map